"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const deprecated_decorator_1 = require("deprecated-decorator");
const schema_middleware_1 = require("./schema.middleware");
const qs = require("qs");
const router_1 = require("./router");
class Tester {
    static install(islandModule) {
        if (Tester.installed)
            return;
        // save previous
        Tester.origin._module = islandModule;
        Tester.origin.method = islandModule.endpointController;
        // install
        islandModule.endpointController = registerer => {
            return target => {
                const e = _.map(target._endpointMethods, (v) => {
                    const splits = v.name.split(' ');
                    return {
                        name: splits[1],
                        options: v.options,
                        method: splits[0]
                    };
                });
                const endpoints = _.groupBy(e, 'method');
                _.mergeWith(Tester.endpoints, endpoints, (target, src) => {
                    if (_.isArray(target))
                        return target.concat(src);
                });
            };
        };
        Tester.installed = true;
    }
    static uninstall() {
        if (!Tester.installed)
            return;
        // revert
        Tester.origin._module.endpointController = Tester.origin.method;
        Tester.origin = {};
        Tester.endpoints = {};
        Tester.installed = false;
    }
    static register(_) {
    }
    static splitUri(uri) {
        const splits = uri.split('?');
        const path = splits[0];
        // https://github.com/spearhead-ea/island/blob/5153829c2f9bcd09e0a53cb82d764702b6518cf0/src/adapters/impl/middlewares/restify-query-parser.ts#L10
        const query = qs.parse(splits[1], { allowDots: true });
        return { path, query };
    }
    static GET(uri, input, sanitizeOnly = false) {
        const splits = Tester.splitUri(uri);
        return Tester.execute('GET', splits.path, {
            query: splits.query,
            body: {},
            session: input.session,
            result: input.result
        }, sanitizeOnly);
    }
    static PUT(uri, input, sanitizeOnly = false) {
        const splits = Tester.splitUri(uri);
        return Tester.execute('PUT', splits.path, {
            query: splits.query,
            body: input.body,
            session: input.session,
            result: input.result
        }, sanitizeOnly);
    }
    static POST(uri, input, sanitizeOnly = false) {
        const splits = Tester.splitUri(uri);
        return Tester.execute('POST', splits.path, {
            query: splits.query,
            body: input.body,
            session: input.session,
            result: input.result
        }, sanitizeOnly);
    }
    static DEL(uri, input, sanitizeOnly = false) {
        const splits = Tester.splitUri(uri);
        return Tester.execute('DEL', splits.path, {
            query: splits.query,
            body: {},
            session: input.session,
            result: input.result
        }, sanitizeOnly);
    }
    static execute(method, path, input, sanitizeOnly) {
        const target = _.find(Tester.endpoints[method], endpoint => {
            const re = router_1.compileURL({ url: endpoint.name });
            const res = router_1.matchURL(re, path);
            if (res)
                input.params = res;
            return res;
        });
        if (!target)
            throw new Error(`invalid uri ${method} ${path}`);
        let results = {};
        for (const name of ['query', 'body', 'session', 'result', 'params']) {
            const schema = target.options.schema[name];
            if (!schema || !input[name])
                continue;
            const sanitized = schema_middleware_1.sanitize(schema.sanitization, input[name]);
            if (!sanitizeOnly) {
                const r = schema_middleware_1.validate(schema.validation, sanitized);
                if (!r.valid)
                    throw new Error(`${sanitized} validation failed ${JSON.stringify(r.error)}`);
            }
            results[name] = sanitized;
        }
        return results;
    }
}
Tester.endpoints = {};
Tester.installed = false;
Tester.origin = {};
__decorate([
    deprecated_decorator_1.default({ version: '2.0.0' }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], Tester, "register", null);
exports.Tester = Tester;
//# sourceMappingURL=index.js.map