"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
const island = require("island");
describe(index_1.Tester.name, () => {
    it('should override endpointController decorator', () => {
        index_1.Tester.install(island);
        const name = 'GET /v2/ping';
        const options = { level: 10 };
        island.endpointController()({ _endpointMethods: [{ name, options }] });
        const endpoints = index_1.Tester.endpoints;
        expect(endpoints['GET'].length).toEqual(1);
        expect(endpoints['GET'][0].method).toEqual('GET');
        expect(endpoints['GET'][0].name).toEqual('/v2/ping');
        expect(endpoints['GET'][0].options).toEqual(options);
        const name2 = 'GET /v2/ping/:id';
        island.endpointController()({ _endpointMethods: [{ name: name2, options }] });
        expect(endpoints['GET'].length).toEqual(2);
        index_1.Tester.uninstall();
        expect(index_1.Tester.endpoints).toEqual({});
        expect(index_1.Tester.origin).toEqual({});
        expect(index_1.Tester.installed).toEqual(false);
    });
    it('should check GET method', () => {
        index_1.Tester.install(island);
        const uri = '/v2/accounts?sp.$lte=100&money.$gt=5000';
        const method = 'GET';
        const name = `${method} ${uri}`;
        const options = { schema: {
                result: {
                    sanitization: {
                        type: 'object',
                        properties: {
                            level: { type: 'number' },
                            hello: { type: 'string' }
                        }
                    },
                    validation: {
                        type: 'object',
                        properties: {
                            level: { type: 'number' },
                            hello: { type: 'string' }
                        }
                    }
                }
            } };
        island.endpointController()({ _endpointMethods: [{ name, options }] });
        const sanitized = index_1.Tester.GET(uri, { result: { hello: 'world', level: '5' } }, true);
        expect(sanitized.result).toEqual({ hello: 'world', level: 5 });
    });
});
//# sourceMappingURL=index.spec.js.map